==> at.bestsolution.rsd.rt.rs.opentracing-1.0.0-20210503.192459-248-sources.jar: at/bestsolution/rsd/rt/rs/opentracing/OpenTracingServerFilter.java <==
package at.bestsolution.rsd.rt.rs.opentracing;

import java.io.IOException;
import java.lang.reflect.Method;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Optional;
import java.util.ServiceLoader;

import javax.ws.rs.container.ContainerRequestContext;
import javax.ws.rs.container.ContainerRequestFilter;
import javax.ws.rs.container.ContainerResponseContext;
import javax.ws.rs.container.ContainerResponseFilter;
import javax.ws.rs.container.ResourceInfo;
import javax.ws.rs.core.Context;
import javax.ws.rs.ext.Provider;

import org.eclipse.microprofile.config.ConfigProvider;

import at.bestsolution.rsd.rt.ServiceProviderSystem;
import at.bestsolution.rsd.rt.opentracing.TraceProvider;
import at.bestsolution.rsd.rt.opentracing.Traced;
import io.opentracing.Scope;
import io.opentracing.SpanContext;
import io.opentracing.Tracer;
import io.opentracing.propagation.Format;
import io.opentracing.propagation.TextMap;
import io.opentracing.tag.Tags;
import io.opentracing.util.GlobalTracer;

@Provider
public class OpenTracingServerFilter implements ContainerRequestFilter, ContainerResponseFilter {
	@Context
	ResourceInfo resourceInfo;

	public static final String SPAN_CONTEXT = "__opentracing_span_context";
	public static final String SPAN = "__opentracing_span";
	private volatile TraceProvider traceProvider;
	
	private String getTraceProviderId() {
		return ConfigProvider.getConfig().getOptionalValue("rsd.opentracing.provider.id", String.class)
				.orElse("jaeger");
	}

	private TraceProvider getActiveTraceProvider() {
		TraceProvider localTraceProvider = traceProvider;
		if (localTraceProvider == null) {
			synchronized (this) {
				localTraceProvider = traceProvider;
				if (localTraceProvider == null) {
					ServiceProviderSystem serviceProviderSystem = ServiceLoader.load(ServiceProviderSystem.class).iterator().next();
					List<TraceProvider> serviceList = serviceProviderSystem.lookupServices(TraceProvider.class);
					if (!serviceList.isEmpty()) {
						String providerId = getTraceProviderId();
						
						Optional<TraceProvider> provider = serviceList.stream()
							.filter( pr -> providerId.equals(pr.getId()))
							.findFirst();
						
						if( provider.isPresent() ) {
							traceProvider = localTraceProvider = provider.get();
						} else {
							traceProvider = localTraceProvider = serviceList.get(0);
						}
					} else {
						traceProvider = new TraceProvider() {
							
							@Override
							public Tracer getTracer() {
								return GlobalTracer.get();
							}
							
							@Override
							public String getId() {
								return "<unknown>";
							}
						};
					}
				}
			}
		}
		
		return localTraceProvider;
	}

	@Override
	public void filter(ContainerRequestContext requestContext) throws IOException {
		Method method = resourceInfo.getResourceMethod();
		Class<?> owner = resourceInfo.getResourceClass();

		boolean shouldTrace = true;
		String operationName = "";

		if (owner.isAnnotationPresent(Traced.class)) {
			Traced annotation = owner.getAnnotation(Traced.class);
			shouldTrace = annotation.value();
			operationName = annotation.operationName();
		}

		if (method.isAnnotationPresent(Traced.class)) {
			Traced annotation = method.getAnnotation(Traced.class);
			shouldTrace = annotation.value();
			operationName = annotation.operationName();
		}

		if (shouldTrace) {
			if (operationName == null || operationName.trim().isEmpty()) {
				operationName = method.getName();
			}
			try {
				initTraceCall(getActiveTraceProvider(), operationName, resourceInfo, requestContext);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	private static void initTraceCall(TraceProvider traceProvider, String operationName, ResourceInfo resourceInfo,
			ContainerRequestContext context) throws Exception {
		Tracer.SpanBuilder spanBuilder = traceProvider.getTracer().buildSpan(operationName);
		spanBuilder = spanBuilder.withTag(Tags.COMPONENT.getKey(), "rest");
		spanBuilder = spanBuilder.withTag("rest.bean", resourceInfo.getResourceClass().getName());
		spanBuilder = spanBuilder.withTag(Tags.SPAN_KIND.getKey(), Tags.SPAN_KIND_SERVER);

		String object = context.getHeaderString("__rsd_trace-span-context");		

		if (object != null) {
			Map<String, String> spanContext = traceProvider.spanContextMap(object);
			SpanContext parentContext = traceProvider.getTracer().extract(Format.Builtin.TEXT_MAP, new TextMap() {

				@Override
				public void put(String key, String value) {
					throw new IllegalStateException("Only called when injecting");
				}

				@Override
				public Iterator<Entry<String, String>> iterator() {
					return spanContext.entrySet().iterator();
				}
			});
			spanBuilder = spanBuilder.asChildOf(parentContext);
		}

		Scope scope = spanBuilder.startActive(true);		
		context.setProperty(SPAN, scope);
		context.setProperty(SPAN_CONTEXT, scope.span().context());
	}

	@Override
	public void filter(ContainerRequestContext requestContext, ContainerResponseContext responseContext)
			throws IOException {
		Scope scope = (Scope) requestContext.getProperty(SPAN);
		if (scope != null) {
			scope.close();
		}
	}
}

==> at.bestsolution.rsd.rt.rs.opentracing-1.0.0-20210722.104733-258-sources.jar: at/bestsolution/rsd/rt/rs/opentracing/OpenTracingServerFilter.java <==
package at.bestsolution.rsd.rt.rs.opentracing;

import java.io.IOException;
import java.lang.reflect.Method;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Optional;
import java.util.ServiceLoader;

import javax.ws.rs.container.ContainerRequestContext;
import javax.ws.rs.container.ContainerRequestFilter;
import javax.ws.rs.container.ContainerResponseContext;
import javax.ws.rs.container.ContainerResponseFilter;
import javax.ws.rs.container.ResourceInfo;
import javax.ws.rs.core.Context;
import javax.ws.rs.ext.Provider;

import org.eclipse.microprofile.config.ConfigProvider;

import at.bestsolution.rsd.rt.ServiceProviderSystem;
import at.bestsolution.rsd.rt.opentracing.TraceProvider;
import at.bestsolution.rsd.rt.opentracing.Traced;
import io.opentracing.Scope;
import io.opentracing.Span;
import io.opentracing.SpanContext;
import io.opentracing.Tracer;
import io.opentracing.propagation.Format;
import io.opentracing.propagation.TextMap;
import io.opentracing.tag.Tags;
import io.opentracing.util.GlobalTracer;

@Provider
public class OpenTracingServerFilter implements ContainerRequestFilter, ContainerResponseFilter {
	@Context
	ResourceInfo resourceInfo;

	public static final String SPAN_CONTEXT = "__opentracing_span_context";
	public static final String SPAN = "__opentracing_span";
	private volatile TraceProvider traceProvider;
	
	private String getTraceProviderId() {
		return ConfigProvider.getConfig().getOptionalValue("rsd.opentracing.provider.id", String.class)
				.orElse("jaeger");
	}

	private TraceProvider getActiveTraceProvider() {
		TraceProvider localTraceProvider = traceProvider;
		if (localTraceProvider == null) {
			synchronized (this) {
				localTraceProvider = traceProvider;
				if (localTraceProvider == null) {
					ServiceProviderSystem serviceProviderSystem = ServiceLoader.load(ServiceProviderSystem.class).iterator().next();
					List<TraceProvider> serviceList = serviceProviderSystem.lookupServices(TraceProvider.class);
					if (!serviceList.isEmpty()) {
						String providerId = getTraceProviderId();
						
						Optional<TraceProvider> provider = serviceList.stream()
							.filter( pr -> providerId.equals(pr.getId()))
							.findFirst();
						
						if( provider.isPresent() ) {
							traceProvider = localTraceProvider = provider.get();
						} else {
							traceProvider = localTraceProvider = serviceList.get(0);
						}
					} else {
						traceProvider = new TraceProvider() {
							
							@Override
							public Tracer getTracer() {
								return GlobalTracer.get();
							}
							
							@Override
							public String getId() {
								return "<unknown>";
							}
						};
					}
				}
			}
		}
		
		return localTraceProvider;
	}

	@Override
	public void filter(ContainerRequestContext requestContext) throws IOException {
		Method method = resourceInfo.getResourceMethod();
		Class<?> owner = resourceInfo.getResourceClass();

		boolean shouldTrace = true;
		String operationName = "";

		if (owner.isAnnotationPresent(Traced.class)) {
			Traced annotation = owner.getAnnotation(Traced.class);
			shouldTrace = annotation.value();
			operationName = annotation.operationName();
		}

		if (method.isAnnotationPresent(Traced.class)) {
			Traced annotation = method.getAnnotation(Traced.class);
			shouldTrace = annotation.value();
			operationName = annotation.operationName();
		}

		if (shouldTrace) {
			if (operationName == null || operationName.trim().isEmpty()) {
				operationName = method.getName();
			}
			try {
				initTraceCall(getActiveTraceProvider(), operationName, resourceInfo, requestContext);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	private static void initTraceCall(TraceProvider traceProvider, String operationName, ResourceInfo resourceInfo,
			ContainerRequestContext context) throws Exception {
		Tracer.SpanBuilder spanBuilder = traceProvider.getTracer().buildSpan(operationName);
		spanBuilder = spanBuilder.withTag(Tags.COMPONENT.getKey(), "rest");
		spanBuilder = spanBuilder.withTag("rest.bean", resourceInfo.getResourceClass().getName());
		spanBuilder = spanBuilder.withTag(Tags.SPAN_KIND.getKey(), Tags.SPAN_KIND_SERVER);

		String object = context.getHeaderString("__rsd_trace-span-context");		

		if (object != null) {
			Map<String, String> spanContext = traceProvider.spanContextMap(object);
			SpanContext parentContext = traceProvider.getTracer().extract(Format.Builtin.TEXT_MAP, new TextMap() {

				@Override
				public void put(String key, String value) {
					throw new IllegalStateException("Only called when injecting");
				}

				@Override
				public Iterator<Entry<String, String>> iterator() {
					return spanContext.entrySet().iterator();
				}
			});
			spanBuilder = spanBuilder.asChildOf(parentContext);
		}

		Span span = spanBuilder.start();
		Scope scope = traceProvider.getTracer().activateSpan(span);
		context.setProperty(SPAN, scope);
		context.setProperty(SPAN_CONTEXT, span.context());
	}

	@Override
	public void filter(ContainerRequestContext requestContext, ContainerResponseContext responseContext)
			throws IOException {
		Scope scope = (Scope) requestContext.getProperty(SPAN);
		if (scope != null) {
			scope.close();
		}
	}
}
